#pragma rtGlobals=1		// Use modern global access method.

// version 2011_05_04
// measure period in reference image/layer
// window:	"FT"+datafolder
// fft image:	datafolder+"_FFT"
// fit image:	fit+datafolder+"_FFT"
// 
// measure period in series
// window:	"PD"+datafolder
// waves:		"PD"+datafolder+"_FFT" - period	
// 			"PE"+datafolder +"_FFT"- error on period	
//			"PD"+datfolder+"_X"

Menu "FFT"
"One_Show_FFT"
"Two_Measure_Period"
"Three_FFT_series"
End

Proc One_Show_FFT()
	FT_show_FFT()	
end

Function FT_show_FFT()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	NVAR fovg=root:$(dfStr):fovg
	NVAR layer=root:$(dfStr):layer
	// image
	String stackStr=SIS_getImageStackName()
	wave w=$stackStr
	// input
	Prompt stackStr, "choose image stack"
	variable ilayer=layer 
	Prompt ilayer, "choose layer"
	variable binning=2
	Prompt binning, "binning", popup "1x1;2x2;4x4;8x8"
	variable FoV=fovg
	Prompt FoV,"field of view (um)"
	doPrompt "input show image FT", stackStr, ilayer, binning, FoV
	// globals
	fovg=FoV
	Variable/G FT_pcsrA
	Variable/G FT_qcsrA
	Variable/G FT_pcsrB
	Variable/G FT_qcsrB
	Variable/G FT_period
	Variable/G FT_errorbar
	Variable/G FT_binning=2
	Variable/G FT_FoV=FoV
	Variable/G FT_angle
	Variable/G FT_errorangle
	make/o W_coef_fft={100000,1E6,NaN,25,NaN,25,0.5}
	// update 
	FT_binning=binning
	//
	if (V_flag==0)
		// get wave
		if (dimSize($stackStr,2)>0)
			ImageTransform/P=(ilayer)/PTYP=0 getPlane $stackStr
		else
			duplicate/o $stackStr M_ImagePlane
		endif
		// do fft
		FT_get_FFT(M_ImagePlane,1,stackStr)	
	endif
end

Function FT_get_FFT(w,disp,nameStr)
	wave w
	variable disp
	String nameStr
	variable np=dimSize(w,0)
	variable nq=dimSize(w,1)
	variable mp=(round(np/2)*2)
	variable mq=(round(np/2)*2)
	String graphStr="FT"+nameStr
	String fftStr=nameStr+"_FFT"
	make/n=(mp,mq)/o $fftStr
	wave wfft = $fftStr
	wfft=w[p][q]
	wavestats/Q wfft
	wfft-=V_avg
	FFT/OUT=3/Z/DEST=tfft/WINF=Hanning wfft // Hanning windowing
	redimension/R tfft
	duplicate/o tfft wfft
	killwaves/z tfft
	SetScale/P x 0,1,"", wfft;DelayUpdate
	SetScale/P y 0,1,"", wfft
	if (disp==1)
		SIS_imageToFront(graphStr,fftStr)
		ModifyImage $fftStr ctab= {0,200000,Grays,0} 
		ModifyGraph width=142
	endif
end

Proc Two_Measure_Period()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// cursors
	String winStr=SIS_getWinName()
	SIS_PauseForCursors(winStr,"Adjust crs A and B")	
	//
	if (stringMatch(num2Str(pcsr(A)),"NaN")!=1 && stringMatch(num2Str(pcsr(B)),"NaN")!=1)
		root:$(dfStr):FT_pcsrA=pcsr(A)
		root:$(dfStr):FT_pcsrB=pcsr(B)
		root:$(dfStr):FT_qcsrA=qcsr(A)
		root:$(dfStr):FT_qcsrB=qcsr(B)
	endif
	// init guess values
	FT_period=root:$(dfStr):FT_period
	FT_errorbar=root:$(dfStr):FT_errorbar
	FT_angle=root:$(dfStr):FT_angle	
	FT_errorangle=root:$(dfStr):FT_errorangle	
	FT_get_Period(dfStr)
	String textStr= "period = "+num2Str(FT_period)+""+num2Str(FT_errorbar)+" nm"+"\r"
	textStr+= "angle = "+num2Str(FT_angle)+""+num2Str(FT_errorangle)+" deg"
	TextBox/C/N=text0/A=MC textStr
end

Function FT_get_Period(dfStr)
	String dfStr
	// call globals
	wave  W_coef_fft=root:$(dfStr):W_coef_fft
	wave  W_sigma=root:$(dfStr):W_sigma
	String fftStr = dfStr+"_FFT"
	String fitStr = "fit_"+dfStr+"_FFT"
	wave wfft=$fftStr
	NVAR FT_pcsrA=root:$(dfStr):FT_pcsrA 
	NVAR FT_pcsrB=root:$(dfStr):FT_pcsrB
	NVAR FT_qcsrA=root:$(dfStr):FT_qcsrA
	NVAR FT_qcsrB=root:$(dfStr):FT_qcsrB
	NVAR FT_period=root:$(dfStr):FT_period
	NVAR FT_angle=root:$(dfStr):FT_angle
	NVAR FT_errorangle=root:$(dfStr):FT_errorangle
	NVAR FT_errorbar=root:$(dfStr):FT_errorbar
	NVAR FT_binning=root:$(dfStr):FT_binning
	NVAR FT_FoV=root:$(dfStr):FT_FoV
	// fit fft image
	duplicate/o W_coef_fft W_coef
	W_coef[2]=(FT_pcsrA+FT_pcsrB)/2
	W_coef[4]=(FT_qcsrA+FT_qcsrB)/2
	Curvefit/H="0000000"/NTHR=0/Q Gauss2D kwCWave=W_coef  wfft[FT_pcsrA,FT_pcsrB][FT_qcsrA,FT_qcsrB] /D 
	// ModifyContour $fitStr autoLevels={*,*,6}
	// calc freq
	variable np=dimsize(wfft,0)
	variable nq=dimsize(wfft,1)
	variable p_fft=W_coef[2]/(np*2) 
	variable q_fft=NaN
	variable nqh=floor(nq/2)
	// two quadrants (both in units of pixels)
	if (W_coef[4]>nqh)
		q_fft=(nq-W_coef[4])/(nq)
	else 
		q_fft=(W_coef[4])/(nq)
	endif
	variable d=sqrt(p_fft^2+q_fft^2)
	// rescale 
	variable scaling = 1000 * FT_FoV * 2^(FT_binning-1) / (870)
	FT_period = scaling/d
	FT_errorbar =  scaling * sqrt(1/W_coef[3]+1/W_coef[5])
	FT_angle= atan(q_fft/p_fft)*180/Pi
	variable dp=W_sigma[2]/np
	variable dq=W_sigma[4]/nq
	FT_errorangle = 180/Pi*1/(1+(q_fft/p_fft)^2)*(abs(dq/p_fft)+abs(q_fft*dp/p_fft^2))

end

Proc Three_FFT_series()
	FT_FFT_series()
end

Function FT_FFT_series()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// stack
	String stackStr=SIS_getImageStackName()
	wave w=$stackStr
	variable np=dimSize(w,0)
	variable nq=dimSize(w,1)
	variable nr=dimSize(w,2)	
	 // globals
	NVAR FT_period=root:$(dfStr):FT_period
	NVAR FT_errorbar=root:$(dfStr):FT_errorbar
	NVAR FT_binning=root:$(dfStr):FT_binning
	NVAR FT_FoV=root:$(dfStr):FT_FoV
	NVAR FT_angle=root:$(dfStr):FT_angle
	NVAR FT_errorangle=root:$(dfStr):FT_errorangle
	NVAR fovg=root:$(dfStr):fovg
	NVAR layer=root:$(dfStr):layer
	 // input
	String wStr
	Prompt wStr, "enter x wave",popup WaveList("*",";","")
	variable type
	variable startfrom=layer
	Prompt startfrom, "start from frame"
	variable endto=nr-1
	Prompt endto, "end to"
	variable FoV=fovg
	Prompt FoV, "field of view (um)"
	doPrompt "left axis input wave" wStr, startfrom, endto, FoV
	FT_FoV=FoV
	
	if (V_flag==0)
		// period waves
		String graphStr="PD"+dfStr
		String pwStr ="PD"+dfStr+"_FFT"
		String pewStr ="PE"+dfStr+"_FFT"
		String awStr ="AN"+dfStr+"_FFT"
		String aewStr ="AE"+dfStr+"_FFT"
		make/n=(nr)/o $pwStr, $pewStr, $awStr, $aewStr
		wave PW=$pwStr
		wave PEW=$pewStr
		wave AW=$awStr
		wave AEW=$aewStr
		PW=NaN
		PEW=NaN
		AW=NaN
		AEW=NaN
		variable i=0
		ImageTransform/P=(i)/PTYP=0 getPlane $stackStr
		wave M_ImagePlane=root:$(dfStr):M_ImagePlane
		do
		if ((i >= startfrom) && (i<= endto))
			ImageTransform/P=(i)/PTYP=0 getPlane $stackStr
			FT_get_FFT(M_ImagePlane,0,stackStr)
			FT_get_Period(stackStr)
			PW[i] =	FT_period
			PEW[i] = FT_errorbar		
			AW[i] =	FT_angle
			AEW[i] = FT_errorangle
			//if the fit is not successful
			if (PW[i]<10)
				PW[i]=NaN
				AW[i]=NaN
			endif
		else
			PW[i] = NaN
		endif
		doUpdate
		i+=1
	while  (i<=nr) 
	
	// display
	String xStr="PD"+dfStr+"_X"
	duplicate/o $wstr, $xStr
	wave wx=$xStr
	String axis1=FFT_graphToFront(graphStr,pwStr,xStr,1)
	String axis2=FFT_graphToFront(graphStr,awStr,xStr,2)
	ModifyGraph mode=3,marker=8
	ModifyGraph mode=4
	// prepare results
	variable axislabel=1
	Prompt axislabel, "axis label", popup "time;temperature;coverage;none"
	DoPrompt "Axis label", axislabel
	if (V_flag==0)
		if (axislabel==1)
			Label $axis1 "\\F'Arial'\\Z18Period (nm)";DelayUpdate
			Label $axis2 "\\F'Arial'\\Z18Angle (deg.)";DelayUpdate
			Label bottom "\\F'Arial'\\Z18Time (s)"
			wx=wx/1000
		endif	
		if (axislabel==2)
			Label $axis1 "\\F'Arial'\\Z18Period (nm)";DelayUpdate
			Label $axis2 "\\F'Arial'\\Z18Angle (deg.)";DelayUpdate
			Label bottom "\\F'Arial'\\Z18Temperature (C)"
		endif	
		if (axislabel==3)
			variable openShutterFrame=0
			Prompt openShutterFrame,"start at frame"
			variable monolayerdef=nr
			Prompt monolayerdef,"reached ML at frame"
			doPrompt "convert time to coverage" openShutterFrame, monolayerdef
			if (V_flag==0)
				variable scaling=wx[monolayerdef]-wx[openShutterFrame]
				variable wx0=wx[openShutterFrame]
				wx-=wx0
				wx/=scaling
			endif
			Label $axis1 "\\F'Arial'\\Z18Period (nm)";DelayUpdate
			Label $axis2 "\\F'Arial'\\Z18Angle (deg.)";DelayUpdate
			Label bottom "\\F'Arial'\\Z18Coverage (ML)"
		endif
		if (axislabel==4)
			Label $axis1 "\\F'Arial'\\Z18Period (nm)";DelayUpdate
			Label $axis2 "\\F'Arial'\\Z18Angle (deg.)";DelayUpdate
			Label bottom "\\F'Arial'\\Z18Other (Arb. Units)"
		endif
		SetAxis $axis1 5,500
		ModifyGraph mode=3
		ModifyGraph fSize=12
		ErrorBars $pwStr Y,wave=($pewStr,$pewStr)
		ErrorBars $awStr Y,wave=($aewStr,$aewStr)
	endif
	endif
End

Function/S FT_returnListFirstElement(str,separator)
	String str
	String separator
	variable n=strlen(str)
	variable j=0
	do
		j+=1;			
	while(j<n && stringMatch(str[j],separator)==0)
	str=str[0,j-1]
	return str
end

Function/S FFT_graphToFront(graphNameStr,waveNameStr,wavexNameStr,axis)
	String graphNameStr
	String waveNameStr
	String wavexNameStr
	variable axis
	String axisStr="Left"+num2Str(axis)
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph/L=$(axisStr) $waveNameStr vs $wavexNameStr
		ModifyGraph freePos($axisStr)=0
		Legend/C/N=text0/A=LT
		ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		ModifyGraph mirror($axisStr)=1
		ModifyGraph width=240,height={Aspect,2}
		ModifyGraph axisEnab(Left1)={0,0.45},axisEnab(Left2)={0.55,1}
		ModifyGraph lblPosMode(Left1)=1,lblPosMode(Left2)=1
		ModifyGraph freePos(Left2)=0
		ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)

			appendToGraph/L=$(axisStr) $waveNameStr vs $wavexNameStr
			ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
			ModifyGraph msize($waveNameStr)=2
			ModifyGraph mirror($axisStr)=1
			ModifyGraph nticks(bottom)=4
			ModifyGraph minor(bottom)=1
			ModifyGraph axisEnab(Left1)={0,0.45},axisEnab(Left2)={0.55,1}
			ModifyGraph lblPosMode(Left1)=1,lblPosMode(Left2)=1
			ModifyGraph freePos(Left2)=0
			ModifyGraph mirror=1
		endif
	endif
	return axisStr
end

// obsolete
// Function time2coverage(waveStr, yStr, xStr, eStr)
	String waveStr, yStr, xStr, eStr
	variable openShutterFrame=0
	Prompt openShutterFrame,"start at frame"
	variable monolayerdef=250
	Prompt monolayerdef,"unit ML at frame"
	doPrompt "convert time to coverage" openShutterFrame, monolayerdef	
	if (V_flag==0)
		String cStr=waveStr+"_C"
		duplicate/o $xStr $cStr
		wave cw=$cStr
		variable scaling=cw[monolayerdef]-cw[openShutterFrame]
		cw-=openShutterFrame
		cw/=scaling
		String graphNameStr="PERC_"+waveStr
		SIS_graphToFront(graphNameStr,yStr,cStr)
	
		ModifyGraph mode=3,marker=8
		ModifyGraph mode=4
		Label left "\\F'Arial'\\Z18Period (nm)";DelayUpdate
		Label bottom "\\F'Arial'\\Z18Coverage (ML)"
		SetAxis left 0,300
		ModifyGraph fSize=12
		ErrorBars yStr Y,wave=(eStr,eStr)
	endif
// End


