#pragma rtGlobals=1		// Use modern global access method.

#include "C:igormacro:SIS:SIS_profiles&ROI"
#include "C:igormacro:SIS:SIS_XMCD"
#include "C:igormacro:SIS:SIS_driftCorrection"
#include "C:igormacro:SIS:SIS_DoniachSunjic"
#include "C:igormacro:SIS:SIS_FFT"
#include "C:igormacro:SIS:SIS_Darkfield"
#include "C:igormacro:SIS:SIS_Particle"
#include "C:igormacro:SIS:SIS_Langmuir"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SPELEEM Image Suite
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// version 2011_05_04
// to do next: /scalebar vertical position, adjust algorithm.
// fix paths
//
//  window name: twoletters+datafoldername, in this way the datafolder is always identified unambiguously
// 	"SI" for single image
//	"MD" for XMC(L)D image
//	"ST" for stack
//	"DC" for drift correction table
//	"AV" for coaddtion image
//	"WF" for waterfall
//	"WP(Q)(C)" waterfall profile
//	"PR" image profile
//	"RO" Roi scan
//	"DS" Donich Sunjic fit
//	"SH" Shirely background removed
//	"FT" Fourier transform
//	"PD" Period waves
//
// wave name: image stack as datafolder, single image as image

Menu "SIS"
	"SIS Panel", SIS_MainPanel()
	"Duplicate Datafolder",  DuplicateStack()
	"Quick Preview", QuickView()
	"Save Graphics", Save_Graphics()
	"Save Data to itx File", SIS_waves2itx("")
	"Roi To LayoutMask", SIS_RoiToLayoutMask()
end

Function SIS_MainPanel()
	String sisStr="SIS"
	Dowindow $sisStr
	SIS_InitGlobals()
	if (V_flag==0)
		NewPanel/N=$(sisStr)/W=(455,0,800,120) as "SpeLeem Image Suite"
		SIS_SisPanelControls()
	else
		DoWindow/F $sisStr
	endif
end

Function SIS_InitGlobals()
	//Create and initalize the globals of SIS
	NewDataFolder/O/S root:SIS
	variable/G scanNpnts
	variable/G layer,valdis,rotangle
	variable/G qLT, pLT, qRB, pRB   //graph marquee
	variable/G darkField=0,normalize=0
	variable/G selectedRow
	String/G scanNameStr=""
	String/G imageTypeStr="tif"
	String/G imagePathStr="D:data"
	String/G lastImageStr=""
	String/G fileNameStr=""
	String/G normImageStr=""
	make/O/T/N=1 normImageList
end

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// path functions
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////  

Function SIS_browsePath(ctrlname): buttoncontrol  //160203
	string ctrlname
	pathInfo imagePath
	SVAR imagePathStr=root:SIS:imagePathStr
	if (V_flag==1)   // path exists already
		newPath/M="please specify a path"/O imagePath imagePathStr	
	endif
	//enter path
	newPath/M="please specify a path"/O imagePath
	SIS_updatePath(1)
end

Function SIS_changePath(ctrlname): buttoncontrol 
	string ctrlname
	newPath/O imagePath SIS_getPath()
      	SIS_updatePath(1)
end

Function/S SIS_getPath()  
	SVAR imagePathStr=root:SIS:imagePathStr
	String s=imagePathStr
	Prompt s, "path to experiment: "
	DoPrompt "please specify a path", s
	return s
End

Function SIS_updatePath(verbose) 	
	variable verbose
	SVAR imagePathStr=root:SIS:imagePathStr
	SVAR scanNameStr=root:SIS:scanNameStr
	pathInfo imagePath
	if (V_flag==1) //ok
		imagePathStr=S_Path
		scanNameStr=SIS_returnFolderNameFromPath(imagePathStr)
		if (verbose==1)
		print "image path has been set to: ",imagePathStr,"scan folder: ", scanNameStr 
		endif
	else
		print "image path has not been set"
	endif
End

Function/S SIS_returnFolderNameFromPath(str)
	String str
	// get rid of last ' 
	variable n=strlen(str)
	if (stringMatch(str[n-1],"'")==1)
		str=str[0,n-2]
	endif
	// get rid of first ' 
	n=strlen(str)
	if (stringMatch(str[0],"'")==1)
		str=str[1, n-1]
	endif
	// get rid of last :
	n=strlen(str)
	if (stringMatch(str[n-1],":")==1)
		str=str[0,n-2]
	endif
	// if *-NNN.* get rid of it
	n=strlen(str)
	variable j=n-1
	do
		j-=1;			
	while(j>=0 && stringMatch(str[j],".")==0)
	if (j>0 && j-5>=0 && stringMatch(str[j-4],"-")==1 && SIS_isNumber(str[j-3,j-1])==1)
		str=str[0,j-5]
	endif
	// extract folder name
	n=strlen(str)
	j=n-1
	do
		j-=1;			
	while(j>=0 && stringMatch(str[j],":")==0)
	str=str[j+1,n-1]
	// debug print "folderRootNameFrom Path output:", str
	return str
end

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// utilities
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Function/S SIS_RemoveForbiddenChar(str)
	String str 
	//remove forbidden char from the name string
	variable n=strlen(str)
	variable i
	for(i=0;i<n;i+=1)
		if ((stringMatch(str[i],"#")==1)|(stringMatch(str[i],"-")==1)|(stringMatch(str[i]," ")==1))
			str=str[0,i-1]+"_"+str[i+1,n-1]
		endif
	endfor
	return str
end

Function/S SIS_RemoveQuotes(str)
	String str
	variable n=strlen(str)
	if (stringMatch(str[n-1],"'")==1)
		str=str[0,n-2]
	endif
	// get rid of first ' 
	n=strlen(str)
	if (stringMatch(str[0],"'")==1)
		str=str[1, n-1]
	endif
	return str
end

Function SIS_isNumber(str)
	String str
	variable n = strlen(str)
	do
		n-=1
	while (0<=n && stringMatch(num2Str(str2num(str[n])),"NaN")==0)
	if (n>=0)
		return 0
	else
		if (stringMatch(str,"")==1)
			return 0 
		else
			return 1
		endif
	endif
end

Function/S SIS_num2ScanNumberStr(p)
	variable p
	if (p<10) 
		return "00"+num2Str(p)
	else
		if (p<100)
			return "0"+num2Str(p)
		else
			return num2Str(p)
		endif
	endif
end

Function/S SIS_returnListFirstElement(str,separator)
	String str
	String separator
	variable n=strlen(str)
	variable j=0
	do
		j+=1;			
	while(j<n && stringMatch(str[j],separator)==0)
	str=str[0,j-1]
	return str
end

Function/S getBcsParameter(inputStr,paramStr)
	String inputStr,paramStr
	variable is_tcp=strsearch(inputStr,"----------",0)
	variable place=2
	if (is_tcp>1)
		place=3
	endif
	variable i1=strsearch(inputStr,paramStr,0)
	String str=inputStr[i1,strlen(inputStr)]
	variable i
	for (i=0;i<place;i+=1)
		i1=strsearch(str,",",0)
		str=str[i1+1,strlen(str)]
	endfor
	i1=strsearch(str,",",0)
	str=str[0,i1-1]
	return str
end

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// data load and save
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Function SIS_LoadUndScan(ctrlname): buttoncontrol
	string ctrlname 
	//Load an image stack image
	SVAR scanNameStr = root:SIS:scanNameStr	
	SVAR imageTypeStr = root:SIS:imageTypeStr
	NVAR darkField=root:SIS:darkField
	String Nwame
	variable step=1, ref
	// scan info
	variable lastdash = strsearch(scanNameStr,"_",Inf,1)
	String nameStr=scanNameStr[0,lastdash-1]
	string scanType = scanNameStr[lastdash+1,strlen(scanNameStr)]
	//make datafolder
	NewDataFolder/S/O root:$(nameStr)
	//
	String scaninfoStr=nameStr+"_trace.txt"
	print "loading data from: ",nameStr+"_"+scanType, ". Auxiliary waves and info from: " ,scaninfoStr
	Loadwave/A/D/G/O/Q/P=imagePath/W scaninfoStr
	lastdash = strsearch(nameStr,"_",Inf,1)
	String scanNumberStr=nameStr[lastdash+1,strlen(nameStr)]
	String phenStr = "Energy"+nameStr[lastdash,strlen(nameStr)]
	String meshStr = "Mesh"+nameStr[lastdash,strlen(nameStr)]
	String currentStr = "Current"+nameStr[lastdash,strlen(nameStr)]
	variable scanNpnts=numpnts($phenStr)
	//
	String graphNameStr
	if (stringmatch(scanType,"UNDSCAN")>0)
		graphNameStr="US"+nameStr
		SIS_graphToFront(graphNameStr,meshStr,phenStr)
			Label bottom "\\F'Arial'\\Z16Photon energy (eV)"
		Label left "\\F'Arial'\\Z16Mesh Readout (V)"
	endif
	if (stringmatch(scanType,"MODSCAN")>0)
		graphNameStr="US"+nameStr
		SIS_graphToFront(graphNameStr,meshStr,currentStr)
			Label bottom "\\F'Arial'\\Z16Current (A)"
		Label left "\\F'Arial'\\Z16Mesh Readout (V)"
	endif
	ModifyGraph margin(left)=50,margin(bottom)=50,margin(top)=25,margin(right)=25;DelayUpdate
	ModifyGraph width=340.157,height=226.772
	ModifyGraph mirror=1
	ModifyGraph fSize=12

	// open beamline file and load parameters
	String filename=nameStr+"_beamline.txt"
	print filename
	Variable refnum
	Open/P=imagePath/R/Z=1 refnum as filename
	if (stringmatch(S_fileName,"")!=1)
		String tmpStr=""
		String textStr=nameStr+"\r"
		// file content
		FreadLine/T=num2char(13) refnum, tmpStr
		ModifyGraph margin(right)=25
		ModifyGraph rgb=(65280,16384,16384)
		// read param
		String bcspointStr
		bcspointStr="Beam energy"
		textStr=textStr+bcspointStr+" = "+getBcsParameter(tmpStr,bcspointStr)+";"+"\r"
		bcspointStr="Beam current"
		textStr=textStr+bcspointStr+" = "+getBcsParameter(tmpStr,bcspointStr)+";"+"\r"
		bcspointStr="Undulator 1 gap"
		textStr=textStr+bcspointStr+" = "+getBcsParameter(tmpStr,bcspointStr)+";"+"\r"
		bcspointStr="Undulator 1 phase"
		textStr=textStr+bcspointStr+" = "+getBcsParameter(tmpStr,bcspointStr)+";"+"\r"
		bcspointStr="Undulator 2 gap"
		textStr=textStr+bcspointStr+" = "+getBcsParameter(tmpStr,bcspointStr)+";"+"\r"
		bcspointStr="Undulator 2 phase"
		textStr=textStr+bcspointStr+" ="+getBcsParameter(tmpStr,bcspointStr)+";"+"\r"
		bcspointStr="Modulator current"
		textStr=textStr+bcspointStr+" = "+getBcsParameter(tmpStr,bcspointStr)+";"
		TextBox/C/N=text1/M/H=12/D={1,1,5}/A=RB textStr
		// calculates maximum
		if (stringmatch(scanType,"UNDSCAN")>0)
			wavestats $meshStr
			wave wphen=$phenStr
			textStr = "undulator scan maximum @ "+num2str(wphen[V_maxloc])	
			TextBox/C/N=text2/M/H=12/D={1,1,5}/A=RT textStr
		endif
		if (stringmatch(scanType,"MODSCAN")>0)
			wavestats $meshStr
			wave wcurr=$currentStr
			textStr = "undulator scan maximum @ "+num2str(wcurr[V_maxloc])	
			TextBox/C/N=text2/M/H=12/D={1,1,5}/A=RT textStr
		endif
	endif
	close refnum
end

Function/S SIS_OpenUmeasureImage(fileName,messageStr)
	String fileName
	String messageStr
	variable singleimage
	variable i
	//Opens the requested image file as a single precision wave and return its name string.
	//In case the image were not found it returns the string "null".
	SVAR imageTypeStr=root:SIS:imageTypeStr
	SVAR fileNameStr=root:SIS:fileNameStr
	NVAR darkfield=root:SIS:darkfield
	NVAR normalize=root:SIS:normalize
	Variable refNum
	// file dialog to select single image
	if (stringmatch(filename,"")==1) 
		singleimage=1
	endif
	Open/P=imagePath/R/M=messageStr/T="."+imageTypeStr/Z=2 refNum as fileName
	if (V_flag<-1)	// check if file exists
		printf "File %s not found in the given folder\r",filename
	elseif (V_flag==-1)
		print "Load file operation cancelled by user"
		return ""
	endif
	//
	String dfStr // name of data folder
	String imageStr
	if (stringMatch(S_fileName,"")!=1) // read image, if exists
		if (singleimage==1) // single image
			dfStr=ParseFilePath(0, S_fileName, ":", 1, 0)
			i=strsearch(dfStr, ".",strlen(dfStr),3)
			dfStr = dfStr[0,i-1]
			imageStr=dfStr
		else // image stack
			dfStr=ParseFilePath(0, S_fileName, ":", 1, 1)
			i=strsearch(dfStr, "_",strlen(dfStr),3)
			if (i>=14)
				dfStr = dfStr[0,i-1]
			endif
			imageStr =  "tmp"
		endif
		// creates new datafolder		
		NewDataFolder/S/O root:$(dfStr)
		//print "data folder and filename?",dfStr, S_fileName //debug
		if ((stringMatch(imageTypeStr,"tif")==1) || (stringMatch(imageTypeStr,"t16")==1 ))
			ImageLoad/T=tiff/O/N=$(imageStr)/Q S_fileName
		elseif (stringMatch(imageTypeStr,"tif32")==1)  
			ImageLoad/T=tiff/O/N=$(imageStr)/Q S_fileName
		elseif ((stringMatch(imageTypeStr,"jpeg")==1)||(stringMatch(imageTypeStr,"JPG")==1))
			ImageLoad/T=JPG/O/N=tmpimage/Q S_fileName
			duplicate /O/R=[][][2,2] tmpimage $imageStr 
  	   		killwaves/z tmpimage	
		elseif ((stringMatch(imageTypeStr,"PNG")==1)||(stringMatch(imageTypeStr,"png")==1))
			ImageLoad/T=RPNG/O/N=$(imageStr)/Q S_fileName
		elseif (stringMatch(imageTypeStr,"ibw")==1)
			Loadwave/H/O S_FileName
			S_FileName=stringFromList(0,S_WaveNames)
			wave tmpWave=$S_Filename
			duplicate/O tmpwave, $imageStr
			killwaves/z tmpwave
		elseif ((stringMatch(imageTypeStr,"DAT")==1)||(stringMatch(imageTypeStr,"dat")==1))	
			SIS_LoadDatImage(S_fileName)
			S_path = S_Filename
		endif
		fileNameStr=S_filename
		variable/G layer=-1					
		close refNum
		// debug
		//variable np=DimSize($imageStr,0)
		//variable nq=DimSize($imageStr,1)
		//print "opened",S_fileName,"as",imageStr,"- Image size: ",np," X ",nq 
		return ImageStr
	else
		return "null"
	endif
end

Function SIS_displaySingleImage(ctrlname): buttoncontrol
	string ctrlname
	//Load a single image
	SVAR lastImageStr=root:SIS:lastImageStr
	SVAR normImageStr=root:SIS:normImageStr
	NVAR darkField=root:SIS:darkField
	NVAR normalize=root:SIS:normalize
	// sets the datafolder to root 
	SetDataFolder root:	
	// open the image, creates and sets the datafolder
       lastImageStr = SIS_OpenUmeasureImage("","Open image")
       variable np=dimsize($lastImageStr,0)
       variable nq=dimsize($lastImageStr,1) 
       // if successful remove darkfield and normalize
	if ((np>0)&&(nq>0))
		// precision: single
		redimension/S  $lastImageStr
		//show and add the controls
		SIS_imageToFront("SI"+lastImageStr,lastImageStr) 		
		SIS_ImageControls()
		doUpdate
		// remove darkfield
		if (darkfield)
			 SIS_darkfield()
		endif
		// normalize
		if (normalize)
			SIS_normalise()
		endif
	endif
	// always restore root as data filder
	SetDataFolder root:SIS
End

Function SIS_darkfield() //(stackStr) // works both with 2D and 3D waves
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave w=$stackStr	
	variable np=dimsize(w,0)
	variable nq=dimsize(w,1)
	variable nr=dimsize(w,2)
	// create a suitable ROI
	make/n=(np,nq)/o/b/u tmpROI
	tmpROI=1
	variable rp=round(np/32)
	variable rq=round(np/32)
	tmpROI[0,rp][0,rq]=0
	tmpROI[np-1-rp,np-1][0,rq]=0
	tmpROI[0,rp][nq-1-rq,nq-1]=0
	tmpROI[np-1-rp,np-1][nq-1-rq,nq-1]=0
	// subtract darkfiled
	if (nr>0)
		ImageStats/P=(0)/R=tmpROI w
	else
		ImageStats/R=tmpROI w
	endif
	w=max(w,V_avg)
	MatrixOp/O wc=w-V_avg
	duplicate/o wc w
	killwaves wc, tmpROI
	return V_avg
end

Function SIS_normalise() //(stackStr) 
	//String stackStr
	// get the normalisation image, stored in SIS
	SetDataFolder root:SIS
	wave/T normImageList=root:SIS:normImageList
	NVAR selectedRow=root:SIS:selectedRow
	variable normImageIndex
	ControlInfo/W=SIS_CorePanel normWaveListCtrl			//read norm wave name from SIS_CorePanel list
	String normStr = normImageList[selectedRow] 
	wave nw=$normStr
	wavestats/Q nw
	variable nw_avg=V_avg
	duplicate/o nw wtmp
	variable antioverflow=0 //20
	wtmp=nw_avg/(nw+antioverflow)
	// df and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave w=$stackStr	
	variable nr= dimSize(w,2)-1
	// wave dimesions must be the same!
	variable np1=dimSize(w,0)
	variable np2=dimSize(nw,0)
	variable nq1=dimSize(w,1)
	variable nq2=dimSize(nw,1)	
	//
	If ((np1==np2) && (nq1==nq2))	
		if (nr>0) //3D case
			MatrixOp/O wc=w[][][0,nr]*wtmp
			duplicate/o wc w
			killwaves wc, wtmp
		else // 2D case
			w=w*wtmp
			killwaves wtmp
		endif
	else
		print "can't perform normalize operation because the normalizatoin image size does not match the target wave"	
	endif
end

Function SIS_makeThisTheNormImage(ctrlname): buttoncontrol	
	String ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	String newStr = "N"+stackStr
	duplicate/o $stackStr root:SIS:$(newStr)
end

Function SIS_LoadDatImage(fileName) 
	String fileName
	killwaves/z tmpImage0, tmpImage
	String GBstr = "GBLoadWave/T={80,80}/W=1/B=1/O/N=tmpImage/P=imagePath/Q"+" "+"\""+fileName+"\"" 
	Execute GBstr  //  load GB
	wave w0=tmpImage0
	variable np=w0[20] 
	variable nq=w0[21]
	DeletePoints 0,196, w0
	redimension/n=(np,nq) w0
	rename tmpImage0 tmpImage
	wave tmpimage
	duplicate/o tmpimage tmp
	wave tmp
	variable nq1 = dimsize(tmpimage,1)
	tmpimage=tmp[p][nq1-1-q]
	killwaves/z tmp
end

Function SIS_imageToFront(graphNameStr,imageNameStr) 
	String graphNameStr
	String imageNameStr
	graphNameStr=SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		NewImage/S=2/N=$(graphNameStr) $imageNameStr
		variable np=dimSize($imageNameStr,0)
		variable nq=dimSize($imageNameStr,1)
		ModifyGraph width=270
		ModifyGraph height={Plan,1,left,top}
		MoveWindow 30,0,300,270
	else
		DoWindow/F $graphNameStr
		MoveWindow 30,0,300,270
	endif
	ShowInfo
End

Function SIS_xyImageToFront(graphNameStr,imageNameStr,pStr,qStr) 
	String graphNameStr
	String imageNameStr
	String pStr,qStr
	Dowindow $(graphNameStr)
	if (V_flag==0)
		display
		Dowindow/C $(GraphNameStr)
		appendImage $imageNameStr vs {$pStr,$qStr}
	else
		DoWindow/F $graphNameStr
	endif
	ModifyGraph width=270
	ModifyGraph height=270
	MoveWindow 30,0,300,270
	ShowInfo
End

Function SIS_waveToFront(graphNameStr,waveNameStr)
	String graphNameStr
	String waveNameStr
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph $waveNameStr 
		Legend/C/N=text0/A=LT
		ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		ModifyGraph width=240,height={Aspect,1}
		ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			appendToGraph $waveNameStr
			ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		endif
	endif
end

Function SIS_graphToFront(graphNameStr,waveNameStr,wavexNameStr)
	String graphNameStr
	String waveNameStr
	String wavexNameStr
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph $waveNameStr vs $wavexNameStr
		Legend/C/N=text0/A=LT
		ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		ModifyGraph width=240,height={Aspect,1}
		ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			appendToGraph $waveNameStr vs $wavexNameStr
			ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		endif
	endif
end

Function SIS_xywaveToLeftAxis(graphNameStr,waveNameStr,wavexNameStr)
	String graphNameStr
	String waveNameStr
	String wavexNameStr
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph $waveNameStr vs $wavexNameStr
		Legend/C/N=text0/A=LT
		ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		ModifyGraph width=240,height={Aspect,1}
		ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			appendToGraph $waveNameStr vs $wavexNameStr
			ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		endif
	endif
end

Function SIS_xywaveToRightAxis(graphNameStr,waveNameStr,wavexNameStr)
	String graphNameStr
	String waveNameStr
	String wavexNameStr
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph/R $waveNameStr vs $wavexNameStr
		Legend/C/N=text0/A=LT
		ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		ModifyGraph width=240,height={Aspect,1}
		ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			appendToGraph/R $waveNameStr vs $wavexNameStr
			ModifyGraph rgb($waveNameStr)=(abs(enoise(65280)),abs(enoise(65280)),abs(enoise(65280)))
		endif
	endif
end

Function SIS_tableToFront(tableNameStr,waveNameStr)
	String tableNameStr
	String waveNameStr
	Dowindow $tableNameStr
	if (V_flag==0)
		//Display
		Edit/N=$(tableNameStr)
		appendToTable $waveNameStr 
	else
		DoWindow/F $tableNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+tableNameStr)
		if (strlen(empty)==0)
			appendToTable $waveNameStr 
		endif
	endif
End

Function SIS_KillStack_ctrl(ctrlname): buttoncontrol
	String ctrlname
	String dfStr=SIS_getDataFolderName()
	String stackStr=SIS_getImageStackName()
	// kill the graph
	String winStr=WinName(0,1)
	DoWindow/k $winStr
	//kill stack
	print "killing stack:", stackStr
	killwaves/z $stackStr
	// kill the datafolder
	KillDataFolder/z root:$(dfStr)
end

Function SIS_LoadAndDisplayImageStack(ctrlname): buttoncontrol
	string ctrlname
	 SIS_Load_UMeasure_Data() 
end
									
Function SIS_Load_UMeasure_Data() 
	//Load an image stack image
	SVAR scanNameStr = root:SIS:scanNameStr	
	SVAR imageTypeStr = root:SIS:imageTypeStr
	NVAR darkField=root:SIS:darkField
	NVAR normalize=root:SIS:normalize
	String fromStr, toStr, afterStr, Nwame
	variable step=1, ref
	variable antioverflow=0
	// get the normalisation image, stored in SIS
	SetDataFolder root:SIS
	wave/T normImageList=root:SIS:normImageList
	variable normImageIndex
	ControlInfo/W=SIS_CorePanel normWaveListCtrl			//read norm wave name from SIS_CorePanel list
	NVAR selectedRow=root:SIS:selectedRow
	String normStr = normImageList[selectedRow]
	// calculate 1/norm
	if (waveexists($normStr) == 1)
		wave nw=$normStr
		wavestats/Q nw
		variable nw_avg=V_avg
		duplicate/o nw wtmp
		wtmp=nw_avg/(nw+antioverflow)
		redimension/S wtmp
		variable np2=dimSize(nw,0)
		variable nq2=dimSize(nw,1)	
	endif
	// scan info
	variable lastdash = strsearch(scanNameStr,"_",Inf,1)
	String nameStr=scanNameStr[0,lastdash-1]
	string scanType = scanNameStr[lastdash+1,strlen(scanNameStr)]
	//
	//make datafolder
	NewDataFolder/S/O root:$(nameStr)
	// print imagePath
	String scaninfoStr=nameStr+"_trace.txt"
	print "loading data from: ",nameStr+"_"+scanType, ". Auxiliary waves and info from: " ,scaninfoStr
	Loadwave/A/D/G/O/Q/P=imagePath/W scaninfoStr
	lastdash = strsearch(nameStr,"_",Inf,1)
	String scanNumberStr=nameStr[lastdash+1,strlen(nameStr)]
	String tmpStr = "ImageNum"+nameStr[lastdash,strlen(nameStr)]
	variable scanNpnts=numpnts($tmpStr)
	variable/G layer, valdis
	String/G selectedwaveStr
	wave/T sequencewavelist
	//
	//load first image in order to dimension the array
	print "number of images in scan: ",scanNpnts 
	tmpStr = nameStr +"_"+SIS_num2ScanNumberStr(0)+".tif" 
	tmpStr =  SIS_OpenUmeasureImage(tmpStr,"")
	
	if  (stringMatch(tmpStr,"")!=1)
    		// initialise the stack
   		variable np=DimSize($tmpStr,0)
   		variable nq=DimSize($tmpStr,1) 
      		make/n=(np,nq,scanNpnts)/O/W $nameStr
  		// create a suitable ROI in order to calc roi
		make/n=(np,nq)/o/b/u tmpROI
		tmpROI=1
		variable rp=round(np/32)
		variable rq=round(np/32)
		tmpROI[0,rp][0,rq]=0
		tmpROI[np-1-rp,np-1][0,rq]=0
		tmpROI[0,rp][nq-1-rq,nq-1]=0
		tmpROI[np-1-rp,np-1][nq-1-rq,nq-1]=0
		ImageStats/R=tmpROI $tmpStr
		variable darkfieldvalue=V_avg
  		// init loop
    		variable j=0
      		do
      			if (j>0)
       			tmpStr=nameStr +"_"+SIS_num2ScanNumberStr(j)+"."+imageTypeStr 
       			tmpStr = SIS_OpenUmeasureImage(tmpStr,"")
       		endif
       		wave wlayer=$tmpStr
 
       		if (normalize==1) 
       			if (((np==np2)&&(nq==nq2))==1)
       				if (darkfield==1)
       					wlayer=max(wlayer,darkfieldvalue)
	       				MatrixOp/o wc=(wlayer-darkfieldvalue)*wtmp
					else
						MatrixOp/o wc=wlayer*wtmp
					endif
       				duplicate/o wc wlayer
       			else
       				print "I can't perform normalize operation because the normalizatoin image size does not match the target wave"	
       			endif
       		else
       			if ((darkfield==1) && (normalize==0))
       				wlayer=max(wlayer,darkfieldvalue)
	       			MatrixOp/o wc=wlayer-darkfieldvalue
       				duplicate/o wc wlayer
       			endif		
       		endif
	       	ImageTransform/P=(j)/PTYP=0/D=$tmpStr setPlane $nameStr 
      		 	killwaves $tmpStr
       		j+=1
       	while (j<scanNpnts && (stringMatch(tmpStr,"")!=1) )
		// display
		if ((np>0)&&(nq>0))
			//show and add the controls
			SIS_imageToFront("ST"+nameStr,nameStr) 
			SIS_SequenceControls(nameStr,scanNpnts) 
			layer=0
			doUpdate
		endif
	endif
end

Function SIS_loadMDImages(ctrlname): buttoncontrol	
	string ctrlname
	// input
	Variable inputType=2
	Prompt inputType, "input type", popup "load all;minus and then plus;"
	doPrompt "Magnetic Dichroism Data",inputType
	if (V_flag==0)	
		if (inputType==1)
			 SIS_Load_UMeasure_Data() 
		endif
		if (inputType==2)
			 SIS_Load_UMeasure_MD_Data() 
		endif
	endif
End

Function SIS_Load_UMeasure_MD_Data() 
	//Load two images into stack 
	SVAR scanNameStr = root:SIS:scanNameStr	
	SVAR imageTypeStr = root:SIS:imageTypeStr
	NVAR darkField=root:SIS:darkField
	NVAR normalize=root:SIS:normalize
	String fromStr, toStr, afterStr, Nwame
	variable step=1, ref
	variable antioverflow=0 //
	// get the normalisation image, stored in SIS
	SetDataFolder root:SIS
	wave/T normImageList=root:SIS:normImageList
	variable normImageIndex
	ControlInfo/W=SIS_CorePanel normWaveListCtrl			//read norm wave name from SIS_CorePanel list
	NVAR selectedRow=root:SIS:selectedRow
	String normStr = normImageList[selectedRow]
	// calculate 1/norm
	if (waveexists($normStr) == 1)
		wave nw=$normStr
		wavestats/Q nw
		variable nw_avg=V_avg
		duplicate/o nw wtmp
		wtmp=nw_avg/(nw+antioverflow)
		redimension/S wtmp
		variable np2=dimSize(nw,0)
		variable nq2=dimSize(nw,1)	
	endif
	// scan info
	variable lastdash = strsearch(scanNameStr,"_",Inf,1)
	String nameStr=scanNameStr[0,lastdash-1]
	string scanType = scanNameStr[lastdash+1,strlen(scanNameStr)]
	//
	//make datafolder
	NewDataFolder/S/O root:$(nameStr)
	// print imagePath
	String scaninfoStr=nameStr+"_trace.txt"
	print "loading data from: ",nameStr+"_"+scanType, ". Auxiliary waves and info from: " ,scaninfoStr
	Loadwave/A/D/G/O/Q/P=imagePath/W scaninfoStr
	lastdash = strsearch(nameStr,"_",Inf,1)
	String scanNumberStr=nameStr[lastdash+1,strlen(nameStr)]
	String phase1Str="phase1_"+scanNumberStr
	String phase2Str="phase2_"+scanNumberStr
	wave wphase1=$phase1Str
	wave wphase2=$phase2Str
	variable scanNpnts=2
	variable/G layer, valdis
	String/G selectedwaveStr
	wave/T sequencewavelist
	//
	//load first image
	String tmpStr = nameStr +"_"+"MINUS"+".tif" 
	tmpStr =  SIS_OpenUmeasureImage(tmpStr,"load the image with negative helicity")
	if (exists(phase1Str) && exists(phase2Str))
		wphase1[0]=-1*abs(wphase1[0])
		wphase2[0]=-1*abs(wphase2[0])
	else
		Make/n=2/O wphase1,wphase2
		wphase1[0]=-1
		wphase2[0]=-1
	endif
	//
	if  (stringMatch(tmpStr,"")!=1)
    		// initialise the stack
   		variable np=DimSize($tmpStr,0)
   		variable nq=DimSize($tmpStr,1) 
      		make/n=(np,nq,scanNpnts)/O/W $nameStr
  		// create a suitable ROI in order to calc roi
		make/n=(np,nq)/o/b/u tmpROI
		tmpROI=1
		variable rp=round(np/32)
		variable rq=round(np/32)
		tmpROI[0,rp][0,rq]=0
		tmpROI[np-1-rp,np-1][0,rq]=0
		tmpROI[0,rp][nq-1-rq,nq-1]=0
		tmpROI[np-1-rp,np-1][nq-1-rq,nq-1]=0
		ImageStats/R=tmpROI $tmpStr
		variable darkfieldvalue=V_avg
  		// loop
    		variable j=0
      		do
      			if (j==1)
				//load second image
				tmpStr = nameStr +"_"+"PLUS"+".tif" 
				tmpStr =  SIS_OpenUmeasureImage(tmpStr,"load the image with positive helicity")
				if (exists(phase1Str) && exists(phase2Str))
					wphase1[j]=abs(wphase1[j])
					wphase2[j]=abs(wphase2[j])
				else
					wphase1[j]=1
					wphase2[j]=1
				endif
       		endif
       		wave wlayer=$tmpStr
       		// norm and darkfield
       		if (normalize==1) 
       			if (((np==np2)&&(nq==nq2))==1)
       				if (darkfield==1)
       					wlayer=max(wlayer,darkfieldvalue)
	       				MatrixOp/o wc=(wlayer-darkfieldvalue)*wtmp
					else
						MatrixOp/o wc=wlayer*wtmp
					endif
       				duplicate/o wc wlayer
       			else
       				print "I can't perform normalize operation because the normalizatoin image size does not match the target wave"	
       			endif
       		else
       			if ((darkfield==1) && (normalize==0))
       				wlayer=max(wlayer,darkfieldvalue)
	       			MatrixOp/o wc=wlayer-darkfieldvalue
       				duplicate/o wc wlayer
       			endif		
       		endif
	       	ImageTransform/P=(j)/PTYP=0/D=$tmpStr setPlane $nameStr 
      		 	killwaves $tmpStr
       		j+=1
       	while (j<scanNpnts && (stringMatch(tmpStr,"")!=1) )
		// display
		if ((np>0)&&(nq>0))
			//show and add the controls
			SIS_imageToFront("ST"+nameStr,nameStr) 
			SIS_SequenceControls(nameStr,scanNpnts) 
			layer=0
			doUpdate
		endif
	endif
End
	
Function SIS_shoMovie(ctrlname): buttoncontrol	
	string ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	//
	variable n=dimSize($stackStr,2)
	SVAR selectedwaveStr=root:$(stackStr):selectedwaveStr
	wave wv=$selectedwaveStr
	NVAR valdis = root:SIS:valdis
	NVAR layer=root:$(stackStr):layer
	variable i=layer	
	variable atstart=layer
	// layer auto adjust
	NVAR layeradjust=root:$(stackStr):layeradjust
	if (dimSize($stackStr,2)>2)
		do
			layer=i
			modifyImage $(stackStr) plane=(i)
			if (layeradjust==1)
				ImageStats/P=(i) $stackStr
				ModifyImage $stackStr ctab={max(0,V_min),V_max,,0}
			endif
			valdis=wv[i]
			ValDisplay vd,value=#num2Str(valdis)
			doUpdate
			i+=1
	while(i<n)
	else // toggle
		for (i=0;i<20;i+=1)
			// show layer one
			layer=1
			modifyImage $(stackStr) plane=(1)
			if (layeradjust==1)
				ImageStats/P=(0) $stackStr
				ModifyImage $stackStr ctab={max(0,V_min),V_max,,0}
			endif
			valdis=wv[1]
			ValDisplay vd,value=#num2Str(valdis)
			doUpdate
			// sho layer 0
			layer=0
			modifyImage $(stackStr) plane=(0)
			if (layeradjust==1)
				ImageStats/P=(0) $stackStr
				ModifyImage $stackStr ctab={max(0,V_min),V_max,,0}
			endif
			valdis=wv[0]
			ValDisplay vd,value=#num2Str(valdis)
			doUpdate
		endfor
	endif
	// go back
	layer=atstart
	modifyImage $(stackStr) plane=(atstart)
	valdis=wv[atstart]
	ValDisplay vd,value=#num2Str(valdis)
	if (layeradjust==1)
		ImageStats/P=(atstart) $stackStr
		ModifyImage $stackStr ctab={max(0,V_min),V_max,,0}
	endif
	doUpdate
End

Function SIS_shoLayer(ctrlname,varNum,varStr,varName): setVariableControl
	String ctrlname
	Variable varNum
	String varStr
	String varName
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	// globals (in folder)
	NVAR layer=root:$(dfStr):layer
	wave/T sequencewavelist=root:$(dfStr):sequencewavelist
	SVAR selectedwaveStr=root:$(dfStr):selectedwaveStr
	wave wv=$selectedwaveStr
	modifyImage $(stackStr) plane=layer
	NVAR valdis = root:SIS:valdis
	valdis=wv[layer]
	ValDisplay vd,value=#num2Str(valdis)
	// layer auto adjust
	NVAR layeradjust=root:$(dfStr):layeradjust
	if (layeradjust==1)
		ImageStats/P=(layer) $stackStr
		ModifyImage $dfStr ctab= {max(0,V_min),V_max,,0}
	endif
	doUpdate
End

Function SIS_makeMovie(ctrlname): buttoncontrol	
	string ctrlname
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	variable n=dimSize($stackStr,2)
	variable i
	SVAR selectedwaveStr=root:$(stackStr):selectedwaveStr
	wave wv=$selectedwaveStr
	NVAR valdis = root:SIS:valdis
	NVAR layer=root:$(stackStr):layer
	// input start, end frame
	variable startfrom=layer
	Prompt startfrom, "start at frame: "
	variable endto=(n-1)
	Prompt endto, "end at frame: "
	variable nframes=5
	Prompt nframes, "movie speed (frame/s): "
	variable scalebar=1
	Prompt scalebar, "scalebar", popup  "no; yes"
	variable textboxinfo=2
	Prompt textboxinfo, "add value textbox", popup "no; yes"
	variable layerinfo=1
	Prompt layerinfo, "add layer textbox", popup "no; yes"
	DoPrompt "make movie", startfrom, endto,nframes, scalebar,textboxinfo,layerinfo
	if (V_flag==0)
		i=startfrom
		String textbox1Str=""
		String textbox2Str=""
		String unitsStr=""
		variable tmpv
		variable timeflag=0
		// ask to save movie
		NewMovie /F=(nframes)/L/O/Z/I as stackStr
		//cosmetics
		ModifyGraph nticks=0
		if (scalebar==2)
			// SIS_addScaleBar()
		endif
		// define units
		if (stringmatch(selectedwaveStr,"pyro*")==1)
			unitsStr=" C"
		endif
		if (stringmatch(selectedwaveStr,"temp*")==1)
			unitsStr=" C"
		endif
		if (stringmatch(selectedwaveStr,"PMCH*")==1)
			unitsStr=" mbar"
		endif
		if (stringmatch(selectedwaveStr,"time*")==1)
			unitsStr=" s"
			timeflag=1
		endif
		if (stringmatch(selectedwaveStr,"STV*")==1)
			unitsStr=" eV"
		endif
		if (stringmatch(selectedwaveStr,"MOBJ*")==1)
			unitsStr=" mA"
		endif
		if (stringmatch(selectedwaveStr,"*NRG*")==1)
			unitsStr=" eV"
		endif
		if (stringmatch(selectedwaveStr,"BeamC*")==1)
			unitsStr=" mA"
		endif		
		// textboxes
		if (layerinfo==2) 
			TextBox/A=LT/C/N=text0 textbox1Str
		endif
		if (textboxinfo==2)
			textbox2Str = textbox2Str + unitsStr
			TextBox/A=RT/C/N=text1 textbox2Str
		endif
		// layer auto adjust
		NVAR layeradjust=root:$(stackStr):layeradjust
		// loop
		do
			layer=i
			modifyImage $(stackStr) plane=(i)
			valdis=wv[i]
			if (layeradjust==1)
				ImageStats/P=(i) $stackStr
				ModifyImage $stackStr ctab= {max(0,V_min),V_max,,0}
			endif
			if (layerinfo==2)
				textbox1Str="# "+ num2str(i) 
				TextBox/C/N=text0 textbox1Str
			endif
			if (textboxinfo==2)
				if (timeflag==1)
					sprintf textbox2Str "%*.*f" 4, 2, wv[i]/1000
				else
					sprintf textbox2Str "%*.*f" 4, 2, wv[i]
				endif
				textbox2Str = textbox2Str + unitsStr
				TextBox/C/N=text1 textbox2Str
			endif
			ValDisplay vd,value=#num2Str(valdis)
			doUpdate
			AddMovieFrame
			i+=1
		while(i<=endto)
		closeMovie
		TextBox/K/N=text0
		TextBox/K/N=text1
		ModifyGraph nticks=9
		ModifyGraph margin=13
		DrawAction delete = 0,100
		// go back
		layer=startfrom
		modifyImage $(stackStr) plane=(startfrom)
		valdis=wv[startfrom]
		ValDisplay vd,value=#num2Str(valdis)
		if (layeradjust==1)
			ImageStats/P=(startfrom) $stackStr
			ModifyImage $stackStr ctab= {max(0,V_min),V_max,,0}
		endif
		doUpdate
	endif
End

Function SIS_addScaleBar()
	//String waveStr=SIS_removeQuotes(SIS_returnListFirstElement(imageNameList("",""),";"))
	//prompt waveStr, "choose image"
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	// input
	variable FoV=6
	Prompt FoV,"field of view (um)"
	variable binning=2
	Prompt binning, "binning", popup "1x1;2x2;4x4;8x8"
	variable scalebarlength=2
	Prompt scalebarlength, "scale bar lenght (um)"
	doPrompt "add Scale Bar" FoV,binning, scalebarlength
	// max scalebar length
	ModifyGraph nticks=0
	if (scalebarlength > 5)
		scalebarlength=5
	endif
	// computes scaling
	variable scaling=870 //scaling (pixels in MCParea)"	
	variable np=dimSize($stackStr,0)
	variable nq=dimSize($stackStr,1)
	variable delta=FoV*( 2^(binning-1)/scaling)
	// draw line and text
	variable length=floor(scalebarlength/delta)
	SetDrawEnv xcoord= top,ycoord= left,linefgc= (65535,0,0),linethick= 3.00;DelayUpdate
	variable xl=0 //40/(2^binning)
	variable xr=45 //240/(2^binning)
	variable xz=36
	DrawLine xl,nq+xr,xl+length,nq+xr
	SetDrawEnv xcoord= top,ycoord= left,textrgb= (65280,0,0),fsize= 24;DelayUpdate
	String scaleBarStr="\\F'Arial'\\Z16"+num2Str(scalebarlength)+" m"
	DrawText xl+0.25*length,nq+xz,scaleBarStr
	ModifyGraph margin(bottom)=50
end

Function SIS_rotate(ctrlname): buttoncontrol
	String ctrlname
	NVAR rotangle=root:SIS:rotangle
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	//
	variable suggestedAngle
	if (stringMatch(num2Str(pcsr(A)),"NaN")!=1 && stringMatch(num2Str(pcsr(B)),"NaN")!=1)
		variable p1=pcsr(A)
		variable p2=pcsr(B)
		variable q1=qcsr(A)
		variable q2=qcsr(B)
		suggestedAngle=round(atan(abs(q2-q1)/abs(p2-p1))*1800/Pi)/10
		rotangle=suggestedAngle
	else
		suggestedAngle=rotangle
	endif
	Prompt suggestedAngle,"rotation angle (deg)" 
	DoPrompt "stack rotate", suggestedAngle
	if (V_flag==0)
		ImageRotate /A=(suggestedAngle)/O $stackStr
		rotangle=suggestedAngle
	endif
end

Function SIS_thres(ctrlname): buttoncontrol
	String ctrlname
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	wave w=$stackStr
	if (dimSize(w,2)>1)
		ImageStats/BEAM $stackStr
	else
		waveStats/Q $stackStr
	endif
	variable lowerThreshold=V_min	
	Prompt lowerThreshold,"new min." 
	variable upperThreshold=V_max	
	Prompt upperThreshold,"new max." 
	DoPrompt "stack threshold", lowerThreshold, upperThreshold
	if (V_flag==0)
		w=max(w,lowerThreshold)
		w=min(w,upperThreshold)
	endif
End

Function SIS_scalebar(ctrlname): buttoncontrol
	String ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	//
	variable scalebar=3
	Prompt scalebar, "graph layout", popup  "plain;no ticks;scalebar;black outside MCP;"
	DoPrompt "Add or Remove Scalebar", scalebar
	// if continue
	if (V_flag == 0)
		if (scaleBar==1)
			ModifyGraph nticks=9
			ModifyGraph margin=13
			DrawAction delete = 0,100
		endif
		if (scaleBar==2)
			ModifyGraph nticks=0
			ModifyGraph margin=13
			DrawAction delete = 0,100
		endif
		if (scaleBar==3)
			DrawAction delete = 0,100
			SIS_addScaleBar()
		endif
		if (scaleBar==4)
			SIS_layoutMask3D()
		endif
	endif
end

Function SIS_savelayer(ctrlname): buttoncontrol
	String ctrlname
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	// input
	variable savegraphics=2
	Prompt savegraphics, "save graphics", popup  "raw data (save layer);save graphics"
	string FileExtList="TIFF;TIFF float;JPEG;PNG", imageTypeStr=""
	Prompt imageTypeStr, "image extension", popup FileExtList
	DoPrompt "Save layer", savegraphics, imageTypeStr
	// give name
	String nameStr = "tmp"
	if (dimsize($stackStr,2)>1)
		NVAR layer=root:$(stackStr):layer
		nameStr = "GR"+stackStr+"_"+SIS_num2ScanNumberStr(layer)
	else
		nameStr = "GR"+stackStr
	endif
	// save
	if (V_flag==0)
		if (savegraphics==2)
			print "command save graphics"
			if ((stringmatch(imageTypeStr,"TIFF")==1) ||  (stringmatch(imageTypeStr,"TIFF float")==1))
				SavePICT/O/T="TIFF"/B=144 as nameStr // save graphics with 2x screen resolution
			endif
			if (stringmatch(imageTypeStr,"JPEG")==1)
				SavePICT/O/Q=1/T="JPEG"/B=144 as nameStr // save graphics with highest quality
			endif
			if (stringmatch(imageTypeStr,"PNG")==1)
				SavePICT/O/E=-5/B=144 as nameStr // save graphics with 2x screen resolution
			endif		
		else
			print "command save raw data"
			SIS_saveSelectedLayer(stackStr,imageTypeStr) // save layer  
		endif
	endif
end

Function SIS_saveSelectedLayer(stackStr,imageTypeStr)
	string stackStr
	string imageTypeStr
	string savenameStr 
	if (dimSize($stackStr,2)>1) // extract layer
		NVAR layer=root:$(stackStr):layer
		ImageTransform/P=(layer)/PTYP=0 getPlane $stackStr 
		wave M_ImagePlane
		savenameStr="ST"+stackStr+"_"+SIS_num2ScanNumberStr(layer)
		duplicate/o M_ImagePlane tmpimage
		killwaves M_ImagePlane
	else // save single image
		duplicate/o $stackStr tmpimage
		savenameStr="SI"+stackStr
	endif
	print imageTypeStr
	if (stringmatch(imageTypeStr,"TIFF")==1)
		Redimension/W/U tmpimage
		ImageSave/D=16/I/O/T="tiff"/U tmpimage as savenameStr
	endif
	if  (stringmatch(imageTypeStr,"TIFF float")==1)
		ImageSave/F/I/O/T="tiff" tmpimage as savenameStr
	endif
	if (stringmatch(imageTypeStr,"JPEG")==1)
		ImageSave/D=40/O/Q=1/T="jpeg" tmpimage as savenameStr // save graphics with highest quality
	endif
	if (stringmatch(imageTypeStr,"PNG")==1)
		Redimension/W/U tmpimage
		ImageSave/D=16/I/O/T="rpng" tmpimage as savenameStr // save graphics with 2x screen resolution
	endif
	// cleanup
	killwaves tmpimage	
End

Function SIS_ImageSubset(): GraphMarquee 
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	NVAR pLT=root:SIS:pLT
	NVAR qLT=root:SIS:qLT
	NVAR pRB=root:SIS:pRB
	NVAR qRB=root:SIS:qRB
	GetMarquee/Z/K left, top
	if (V_Flag == 0)
		Print "There is no marquee"
	else
		pLT=round(V_top)
		pRB=round(V_bottom)
		qLT=round(V_left)
		qRB=round(V_right)
		SIS_redimensionImage(stackStr, pLT, pRB, qLT, qRB)
	endif
End

Function SIS_previousSubset(): GraphMarquee  
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	NVAR pLT=root:SIS:pLT
	NVAR qLT=root:SIS:qLT
	NVAR pRB=root:SIS:pRB
	NVAR qRB=root:SIS:qRB
	SIS_redimensionImage(stackStr, pLT, pRB, qLT, qRB)
End

 Function SIS_redimensionImage(imageStr,row1,row2,col1,col2)  
 	String imageStr
 	variable row1,row2,col1,col2
 	variable nRows,nCols, nLayers
 	wave imagewave=$imageStr
 	nCols=dimSize(imagewave,0)
 	nRows=dimSize(imagewave,1)
 	nLayers=dimSize(imagewave,2)
 	if (row1>nRows)
		row1=nRows
	endif
	if (row2>nRows)
		row2=nRows
	endif
	if (col1>nCols)
		col1=nCols
	endif
	if (row2>nCols)
		col2=nCols
	endif
	variable c1,c2,r1,r2
	c1=min(col1,col2)
	c2=max(col1,col2)
	r1=min(row1,row2)
	r2=max(row1,row2)
	DeletePoints/M=0   (c2+1),(nCols-1), imagewave
 	DeletePoints/M=0   0,(c1-1), imagewave
 	DeletePoints/M=1   (r2+1),(nRows-1), imagewave
 	DeletePoints/M=1   0,(r1-1), imagewave
 	if (nLayers <= 1)
 		redimension/N=(c2-c1,r2-r1) imagewave
 	else
 		redimension/N=(c2-c1,r2-r1,nLayers) imagewave
 	endif
 end	

Function SIS_prevsubset(ctrlname): buttoncontrol
	String ctrlname
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	NVAR pLT=root:SIS:pLT
	NVAR qLT=root:SIS:qLT
	NVAR pRB=root:SIS:pRB
	NVAR qRB=root:SIS:qRB
	SIS_redimensionImage(stackStr, pLT, pRB, qLT, qRB)	
end

Function SIS_RoiToLayoutMask()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	if (exists("M_RoiMask")==1)
		duplicate/o M_RoiMask root:SIS:M_layoutMask
	endif
end

Function SIS_removeBkg(ctrlname): buttoncontrol
	String ctrlname
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	wave w=$stackStr
	if (exists("M_RoiMask")==1)
		ImageStats/R=M_RoiMask w
		duplicate/o M_RoiMask tmpMask
		tmpMask-=1
		ImageRemoveBackground/R=tmpMask/O/P=3 w
		killwaves/z tmpMask
		w+=V_avg
	endif
End

Function SIS_imageLayout(ctrlname): buttoncontrol
	String ctrlname
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()	
	wave w=$stackStr
	variable np=dimSize(w,0)
	variable nq=dimSize(w,1)
	variable nr=dimSize(w,2)
	if (exists("root:SIS:M_LayoutMask")==1)
		variable npm=dimsize(root:SIS:M_LayoutMask,0)
		variable nqm=dimsize(root:SIS:M_LayoutMask,1)
		if ((np==npm)&&(nq==nqm))
			if (nr==0)
				ImageStats/R=root:SIS:M_LayoutMask w
				variable mini=max(V_min,0)
				SIS_layoutMask(w,root:SIS:M_LayoutMask,0)
			endif
		endif
	endif
End

Function SIS_layoutMask(w1,w2,k)
	wave w1
	wave w2
	variable k
	w1=(1-w2)*w1+abs(w2)*k
end

Function SIS_layoutMask3D()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// stack
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	variable nr=dimSize($stackStr,2)
	variable i=0
	if (exists("root:SIS:M_layoutMask"))
		if (nr>1)
			for (i=0;i<nr;i+=1)
				ImageTransform/P=(i)/PTYP=0 getPlane $stackStr
				duplicate/o M_ImagePlane wtmp
				SIS_layoutMask(wtmp,root:SIS:M_layoutMask,0)
				ImageTransform/P=(i)/PTYP=0/D=wtmp setPlane $stackStr
			endfor
		endif
		if (nr==0)
			SIS_layoutMask($stackStr,root:SIS:M_layoutMask,0)
		endif	
	endif
end

Function/S SIS_getWinName()
	String winStr=WinName(0,1)
	winStr= SIS_removeQuotes(winStr)
	return winStr
end 

Function/S SIS_getDataFolderName()
	String winStr=WinName(0,1)
	winStr= SIS_removeQuotes(winStr)
	variable i=strsearch(winStr,"2",0)	
	String dfStr
	if (i>0)
		dfStr=winStr[i,strlen(winStr)]
	else
		 dfStr=winStr[2,strlen(winStr)] // deals with names
	endif
	return dfStr
end

Function/S SIS_getImageStackName()
	String stackStr=StringFromList(0,imageNameList("",""),";")
	stackStr=SIS_removeQuotes(stackStr)
	return stackStr
end

Proc DuplicateStack()
	SIS_cloneDataFolder()
End

Proc QuickView()
	SIS_quickView()
end
 
Function SIS_cloneDataFolder()
	// data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// make new df
	String newdfStr=dfStr+"0"
	NewDataFolder/O root:$(newdfStr)
	// copy all 1D waves into it
	make/n=1/o/T sequencewavelist
	String tempListStr=WaveList("*",";","DIMS:1")
	variable n=itemsInlist(tempListStr)+stringmatch(tempListStr,"")
	redimension/N=(n) sequencewavelist //update the text wave linked to the listbox
	sequencewavelist=stringFromList(p,tempListStr) 
	variable i=0
	for (i=0;i<n;i+=1)
		duplicate/o $sequencewavelist[i] root:$(newdfStr):$sequencewavelist[i]
	endfor
	// duplicate image or stack
	String stackStr=SIS_getImageStackName()
	duplicate/o $stackStr root:$(newdfStr):$newdfStr
	variable nr=dimSize($stackStr,2)
	//set df
	SetDataFolder root:$(newdfStr)
	variable/G layer=0, valdis
	String/G selectedwaveStr
	wave/T sequencewavelist
	if (nr>1)
		SIS_imageToFront("ST"+newdfStr,newdfStr) 
		SIS_SequenceControls(newdfStr,nr)
	endif
	if (nr==0)
		SIS_imageToFront("SI"+newdfStr,newdfStr) 		
		SIS_ImageControls()
	endif
End

Function SIS_quickView()
	String ctrlname
	// set data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	// stack
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	variable nr=dimSize($stackStr,2)
	// show
	variable ilayer=0
	ImageTransform/P=(0)/PTYP=0 getPlane $stackStr
	wave M_imagePlane
	do
		if (ilayer==0)
			SIS_imageToFront("PREVIEW","M_imagePLane")
		endif
		ImageTransform/P=(ilayer)/PTYP=0 getPlane $stackStr
		doUpdate
		ilayer+=2
	while (ilayer < nr)
	Dowindow/K PREVIEW
end

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Panels, buttons and controls
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Function SIS_SisPanelControls()
	SetDataFolder root:SIS
	Button browseImagePathButton,pos={5,5},size={95,18},proc=SIS_browsePath,title="browse path"
	Button loadImageButton,pos={105,5},size={95,18},proc=SIS_displaySingleImage,title="load image"
	Button loadUndScanButton,pos={205,5},size={95,18},proc=SIS_loadUndScan,title="undulator scan"
	// second line
	Button loadMDImagesButton,pos={5,25},size={95,18},proc=SIS_loadMDImages,title="load MCD/MLD"
	Button loadSequenceButton,pos={105,25},size={95,18},proc=SIS_LoadAndDisplayImageStack,title="load sequence"
	PopupMenu imageTypePopup,pos={205,25},size={95,18},proc=SIS_SetImageType,title="image type: "
	PopupMenu imageTypePopup,mode=3,popvalue="tif",value= #"\"tif;jpg;png;dat;ibw\""
	// third
	CheckBox Darkfield,pos={5,77},size={60,15},title="Darkfield"
	CheckBox Darkfield,variable= root:SIS:darkField
	CheckBox Normalize,pos={5,55},size={60,15},title="Normalize"
	CheckBox Normalize,variable= root:SIS:normalize
	ListBox NormWaveListCtrl,pos={105,50},size={145,58},proc=SIS_ListImageNorm_ctrl
	ListBox NormWaveListCtrl,listWave=root:SIS:normImageList,mode= 2,selRow= 4
EndMacro
	
Function SIS_ImageControls()
	ControlBar 70
	// first line
	Button scalebarButton,pos={5,5},size={90,18},proc=SIS_scalebar,title="scalebar"
	Button prevsubsetButton,pos={100,5},size={90,18},proc=SIS_prevsubset,title="prev subset"
	Button savelayerButton,pos={195,5},size={90,18},proc=SIS_savelayer,title="save image"
	Button killButton,pos={290, 5},size={90,18},proc=SIS_KillStack_ctrl,title="kill"
	// second line
	Button rotateButton,pos={5,25},size={90,18},proc=SIS_rotate,title="rotate"
	Button thresButton,pos={100,25},size={90,18},proc=SIS_thres,title="threshold"
	Button layoutButton,pos={195,25},size={90,18},proc=SIS_imageLayout, title="layout"
	Button removeBkgButton,pos={290,25},size={90,18},proc=SIS_removeBkg,title="removeBkg"
	Button tonormButton,pos={5,45},size={90,18},proc=SIS_makeThisTheNormImage,title="make it norm"
end

function SIS_SequenceControls(dfStr,scanNpnts) 
	String dfStr
	variable scanNpnts
	ControlBar 90
	SetDataFolder root:$(dfStr)
	NVAR layer=root:$(dfStr):layer
	NVAR valdis=root:SIS:valdis
	SVAR selectedwaveStr=root:$(dfStr):selectedwaveStr
	// set layer
	SetVariable sl,pos={5,5},size={90,18},proc=SIS_shoLayer,title="layer"
	SetVariable sl,font="Arial",limits={0,scanNpnts-1,1},value=root:$(dfStr):layer
	// wave list
	make/n=10/o/T sequencewavelist
	wave/T sequencewavelist=root:$(dfStr):sequencewavelist
	String tempListStr=WaveList("*",";","DIMS:1")
	variable n=itemsInlist(tempListStr)+stringmatch(tempListStr,"")
	redimension/N=(n) sequencewavelist //update the text wave linked to the listbox
	sequencewavelist=stringFromList(p,tempListStr) 
	selectedwaveStr=stringFromList(1,tempListStr,";")
	ListBox wlc,pos={100,5},size={100,78},proc=SIS_sequencewavelist
	ListBox wlc,listWave=sequencewavelist,mode= 2,selRow= 1
	// val display
	ValDisplay vd, pos={205,5} , size={185,18}, frame=2, font="Helvetica",title="param value" ,value= #"foo"
	ValDisplay vd,format="%.2f",limits={0,0,0},barmisc={0,1000}, disable=2
	ValDisplay vd, value=0
	// first colum
	Button shomovieButton,pos={5,25},size={90,18},proc=SIS_shomovie,title="run movie"
	Button makeMovieButton,pos={5,45},size={90,18},proc=SIS_makemovie,title="make movie"
	// visualisation checkbox
	Variable/G layeradjust=0
	CheckBox adjcb,pos={5,68},size={90,18},title="layer adjust"
	CheckBox adjcb, variable= root:$(dfStr):layeradjust	
	// 3rd col
	Button prevsubsetButton,pos={205,25},size={90,18},proc=SIS_prevsubset,title="prev subset"
	Button rotateButton,pos={205,45},size={90,18},proc=SIS_rotate,title="rotate"
	Button thresButton,pos={205,65},size={90,18},proc=SIS_thres,title="threshold"
	// 4th col
	Button layoutButton,pos={300,25},size={90,18},proc=SIS_scalebar,title="layout&scalebar"
	Button savelayerButton,pos={300,45},size={90,18},proc=SIS_savelayer,title="save layer"
	Button killButton,pos={300,65},size={90,18},proc=SIS_KillStack_ctrl,title="kill"
end

Function SIS_SetImageType(ctrlName,popNum,popStr): PopupMenuControl //160203
	String ctrlName
	Variable popNum
	String popStr
	SVAR imageTypeStr = root:SIS:ImageTypeStr
      imageTypeStr=popStr
      print "image type has been set to: ",imageTypeStr
End

Function SIS_sequencewavelist(ctrlname,row,col,event): ListBoxControl
	String ctrlname
	Variable row
	Variable col
	Variable event	//1=mouse down, 2=up, 3=dbl click, 4=cell select with mouse or keys
					//5=cell select with shift key, 6=begin edit, 7=end				
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	//		
	wave/T sequencewavelist=root:$(dfStr):sequencewavelist
	SVAR selectedwaveStr=root:$(dfStr):selectedwaveStr
	// sets the variable
	selectedwaveStr=sequencewavelist[row]
	// adjust valdisplay indicator
	wave wv=$selectedwaveStr
	NVAR valdis = root:SIS:valdis
	NVAR layer=root:$(dfStr):layer
	valdis=wv[layer]
	ValDisplay vd,value=#num2Str(valdis)
	doUpdate
End

Function SIS_ListImageNorm_ctrl(ctrlName,row,col,event): ListBoxControl
	String ctrlName
	Variable row
	Variable col
	Variable event	//1=mouse down, 2=up, 3=dbl click, 4=cell select with mouse or keys
					//5=cell select with shift key, 6=begin edit, 7=end
	SVAR normImageStr=root:SIS:normImageStr
	wave/T normImageList=root:SIS:normImageList
	NVAR selectedRow=root:SIS:selectedRow
	String waveListStr
	variable num
	// updates the list
	setDataFolder root:SIS
	waveListStr=wavelist("N*",";","MINCOLS:2,MINROWS:2,MAXLAYERS:0")  	//selection of single images
	num=itemsInlist(waveListStr)+stringmatch(waveListStr,"")
	redimension/N=(num) normImageList									 //update the text wave linked to the listbox
	normImageList=stringFromList(p,waveListStr)  
	normImageStr=normImageList[row]
	selectedRow=row
End

Proc Save_Graphics()
	SIS_savegraphics()
end

Function SIS_savegraphics()
	String graphStr
	variable widthmode=1
	prompt widthmode "graph aspect ratio", popup "no change;square;rectangular (wide);rectangular (tall);"
	string FileExtList="TIFF;JPEG;PNG", imageTypeStr=""
	Prompt imageTypeStr, "image extension", popup FileExtList
	DoPrompt "Select graph width", widthmode, imageTypeStr
	if (widthmode==2)
	ModifyGraph width=240
		ModifyGraph height=240
	endif
	if (widthmode==3)
		ModifyGraph width=480,height={Aspect,0.5}
	endif
	if (widthmode==4)
		ModifyGraph width=240,height={Aspect,2}
	endif
	graphStr=WinName(0,1)
	newpath/C/M="enter path" savegraphics
	if (stringmatch(imageTypeStr,"TIFF")==1)
		SavePICT/P=savegraphics/T="TIFF"/B=288 as graphStr+"."+imageTypeStr // save graphics with 2x screen resolution
	endif
	if (stringmatch(imageTypeStr,"JPEG")==1)
		SavePICT/P=savegraphics/O/Q=1/T="JPEG"/B=288 as graphStr+"."+imageTypeStr // save graphics with highest quality
	endif
	if (stringmatch(imageTypeStr,"PNG")==1)
		SavePICT/P=savegraphics/O/E=-5/B=288 as graphStr+"."+imageTypeStr // save graphics with 2x screen resolution
	endif			
end

//Proc SaveWaveItx()
	//String dfStr=SIS_getDataFolderName()
	//SetDataFolder root:$(dfStr)
	//String tmpStr=WaveList("*", ";","WIN:") 
	//newpath/C/M="enter path"/O savewaves
	//if (V_flag==0)
	//	Save/O/T/W/I/B/P=savewaves tmpStr
	//endif
//end

Function SIS_waves2itx(ctrlname): buttoncontrol
	String ctrlname
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String tmpStr=WaveList("*", ";","WIN:") 
	newpath/C/M="enter path"/O savewaves
	if (V_flag==0)
		Save/O/T/W/I/B/P=savewaves tmpStr
	endif
end

// usage
// (SIS_PauseForCursors(winStr,"message")
// if (strlen(CsrWave(A))>0 && strlen(CsrWave(B))>0) // Cursors on trace?
Function SIS_PauseForCursors(graphName,messageStr)
	String graphName
	String messageStr
	DoWindow/F $graphName // Bring graph to front
	if (V_Flag == 0) // Verify that graph exists
		Abort "UserCursorAdjust: No such graph."
		return -1
	endif

	NewPanel /K=2 /W=(200,400,400,500) as "Pause for Cursor"
	DoWindow/C tmp_PauseforCursor // Set to an unlikely name
	AutoPositionWindow/E/M=1/R=$graphName // Put panel near the graph
	DrawText 21,20,messageStr
	DrawText 21,40,"and click Continue."
	Button button0,pos={80,58},size={92,20},title="Continue"
	Button button0,proc=SIS_continue
	Variable didAbort= 0
	PauseForUser tmp_PauseforCursor,$graphName
	return didAbort
End

Function SIS_continue(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K tmp_PauseforCursor // Kill self
End